'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _findUp = require('find-up');

var _findUp2 = _interopRequireDefault(_findUp);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _convertSeconds = require('convert-seconds');

var _convertSeconds2 = _interopRequireDefault(_convertSeconds);

var _validFile = require('valid-file');

var _validFile2 = _interopRequireDefault(_validFile);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _pendel = require('pendel');

var _pendel2 = _interopRequireDefault(_pendel);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _promptsJs = require('./prompts.js');

var TimeCard = (function () {
	/**
  * Initialize a new Timecard
  *
  * @param {Object} options
  * @api public
  */

	function TimeCard(options) {
		_classCallCheck(this, TimeCard);

		this.options = (0, _objectAssign2['default'])({}, options);

		var pkgRoot = _findUp2['default'].sync('package.json');
		var root = pkgRoot ? _path2['default'].dirname(pkgRoot) : process.cwd();
		this.filepath = this.options.filepath ? _path2['default'].join(this.options.filepath, '.timecard.json') : _path2['default'].join(root, '.timecard.json');
		this.name = process.cwd().split(_path2['default'].sep).pop();
		this.prompt = typeof this.options.prompt === 'boolean' ? this.options.prompt : true;
		this.clockoutIsPending = false;
		this.pendingClockoutIndex = null;
		this.totalSeconds = 0;
		this.timecardData = null;
		this.shifts = [];
	}

	/**
  * Create a blank timecard on `timecard new`
  *
  */

	_createClass(TimeCard, [{
		key: 'create',
		value: function create() {
			var self, msg;
			return _regeneratorRuntime.async(function create$(context$2$0) {
				while (1) switch (context$2$0.prev = context$2$0.next) {
					case 0:
						self = this;
						msg = _chalk2['default'].gray(self.filepath) + ' \n ' + self.messages.createdNewTimeCard;
						return context$2$0.abrupt('return', new _Promise(function (resolve) {
							if (self.prompt === true) {
								var questions = (0, _promptsJs.createPrompt)();

								// First check to see if the file already exists
								if (_validFile2['default'].sync(self.filepath) === true) {
									questions = (0, _promptsJs.eraseThenCreatePrompt)();
								}

								_inquirer2['default'].prompt(questions, function (answers) {
									if (typeof answers.eraseCard === 'undefined' || answers.eraseCard === true) {
										self.writeTimeCard(self.blankTimecard(answers.projectName)).then(function () {
											resolve(msg);
										});
									}
								});
							} else {
								self.writeTimeCard(self.blankTimecard(self.name)).then(function () {
									resolve(msg);
								});
							}
						}));

					case 3:
					case 'end':
						return context$2$0.stop();
				}
			}, null, this);
		}

		/**
   * Clockin.
   *
   */

	}, {
		key: 'clockin',
		value: function clockin() {
			var self;
			return _regeneratorRuntime.async(function clockin$(context$2$0) {
				var _this = this;

				while (1) switch (context$2$0.prev = context$2$0.next) {
					case 0:
						self = this;
						return context$2$0.abrupt('return', new _Promise(function callee$2$0(resolve, reject) {
							var date, tc;
							return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
								while (1) switch (context$3$0.prev = context$3$0.next) {
									case 0:
										if (!(_validFile2['default'].sync(self.filepath) === false)) {
											context$3$0.next = 4;
											break;
										}

										reject(self.errors.noTimeCardFoundForClockin);
										context$3$0.next = 7;
										break;

									case 4:
										context$3$0.next = 6;
										return _regeneratorRuntime.awrap(self.processTimeCardData());

									case 6:

										if (self.clockoutIsPending) {
											reject(self.errors.clockOutIsPending);
										} else {
											date = new Date().toString();
											tc = {
												id: self.shifts.length,
												date: date.slice(0, 15),
												startTime: date.slice(16, 24)
											};

											self.timecardData.shifts.push(tc);

											self.writeTimeCard(JSON.stringify(self.timecardData, null, 2)).then(function () {
												resolve(self.messages.successfulClockin);
											});
										}

									case 7:
									case 'end':
										return context$3$0.stop();
								}
							}, null, _this);
						}));

					case 2:
					case 'end':
						return context$2$0.stop();
				}
			}, null, this);
		}

		/**
   * Clockout.
   *
   */

	}, {
		key: 'clockout',
		value: function clockout() {
			var self;
			return _regeneratorRuntime.async(function clockout$(context$2$0) {
				var _this2 = this;

				while (1) switch (context$2$0.prev = context$2$0.next) {
					case 0:
						self = this;
						return context$2$0.abrupt('return', new _Promise(function callee$2$0(resolve, reject) {
							return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
								while (1) switch (context$3$0.prev = context$3$0.next) {
									case 0:
										if (!(_validFile2['default'].sync(self.filepath) === false)) {
											context$3$0.next = 4;
											break;
										}

										reject(self.errors.noTimeCardFoundForClockout);
										context$3$0.next = 7;
										break;

									case 4:
										context$3$0.next = 6;
										return _regeneratorRuntime.awrap(self.processTimeCardData());

									case 6:

										if (self.clockoutIsPending) {
											(function () {
												var date = new Date().toString();

												self.shifts[self.pendingClockoutIndex].endTime = date.slice(16, 24);

												// report the last 'shift' time and the total time to the console.
												var shiftSeconds = (0, _pendel2['default'])(self.shifts[self.pendingClockoutIndex].startTime, self.shifts[self.pendingClockoutIndex].endTime).seconds;
												var clockOutSummary = self.clockoutSummary(shiftSeconds, self.totalSeconds + shiftSeconds);

												self.totalSeconds += shiftSeconds;
												self.timecardData.totals = (0, _convertSeconds2['default'])(self.totalSeconds);

												self.writeTimeCard(JSON.stringify(self.timecardData, null, 2)).then(function () {
													resolve(clockOutSummary + '\n ' + self.messages.successfulClockout);
												});
											})();
										} else {
											reject(self.errors.noClockInFound);
										}

									case 7:
									case 'end':
										return context$3$0.stop();
								}
							}, null, _this2);
						}));

					case 2:
					case 'end':
						return context$2$0.stop();
				}
			}, null, this);
		}

		/**
   * Print the timecard data to the console.
   *
   */

	}, {
		key: 'printTimecard',
		value: function printTimecard() {
			var self, output;
			return _regeneratorRuntime.async(function printTimecard$(context$2$0) {
				while (1) switch (context$2$0.prev = context$2$0.next) {
					case 0:
						self = this;
						context$2$0.next = 3;
						return _regeneratorRuntime.awrap(self.processTimeCardData());

					case 3:
						output = self.prettyPrintHeader();

						self.timecardData.shifts.forEach(function (item) {
							output += self.prettyPrintEntry(item);
						});
						output += self.messages.prettyPrintBorder;
						output += self.summary(self.totalSeconds);
						return context$2$0.abrupt('return', _Promise.resolve(output));

					case 8:
					case 'end':
						return context$2$0.stop();
				}
			}, null, this);
		}
	}]);

	return TimeCard;
})();

exports['default'] = TimeCard;

(0, _objectAssign2['default'])(TimeCard.prototype, require('./utils.js'));
(0, _objectAssign2['default'])(TimeCard.prototype, require('./messages.js'));
module.exports = exports['default'];

// process timecard to see if a clockout is pending.

// process timecard to see if a clockout is pending.