'use strict';

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.processTimeCardData = processTimeCardData;
exports.writeTimeCard = writeTimeCard;
exports.getTimeCard = getTimeCard;
exports.blankTimecard = blankTimecard;
exports.checkForValidTimecard = checkForValidTimecard;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _pendel = require('pendel');

var _pendel2 = _interopRequireDefault(_pendel);

var _lodashSortby = require('lodash.sortby');

var _lodashSortby2 = _interopRequireDefault(_lodashSortby);

/**
 * Read the json file with the timecard data and set Timecard state.
 *
 * @returns {*} Promise
 */

function processTimeCardData() {
	var self = this;

	return new _Promise(function (resolve, reject) {
		self.getTimeCard().then(function (data) {
			self.timecardData = JSON.parse(data);
			self.name = self.timecardData.name;
			var shifts = self.timecardData.shifts;

			// reset to zero each time the timecard data is read
			self.totalSeconds = 0;

			if (shifts.length > 0) {
				self.shifts = (0, _lodashSortby2['default'])(shifts, function (shift) {
					return shift.id;
				});

				var lastShift = self.shifts[self.shifts.length - 1];
				if (lastShift.startTime && !lastShift.endTime) {
					self.clockoutIsPending = true;
					self.pendingClockoutIndex = lastShift.id;
				}
				self.shifts.forEach(function (item) {
					if (item.startTime && item.endTime) {
						self.totalSeconds += (0, _pendel2['default'])(item.startTime, item.endTime).totalSeconds;
					}
				});
				resolve(self.shifts);
			} else {
				self.shifts = [];
				resolve(self.timecardData);
			}
		})['catch'](function (err) {
			reject(err);
		});
	});
}

/**
 * Physically write the timecard data to the json file.
 *
 * @param {object} data - the json data to write
 */

function writeTimeCard(data) {
	var self = this;

	return new _Promise(function (resolve, reject) {
		_fs2['default'].writeFile(self.filepath, data, function (err) {
			if (err) {
				reject(err);
			} else {
				resolve();
			}
		});
	});
}

/**
 * Read the .timecard.json file from disk and return its contents
 *
 * @returns {*} Promise
 */

function getTimeCard() {
	var self = this;

	return new _Promise(function (resolve, reject) {
		_fs2['default'].readFile(self.filepath, 'utf8', function (err, data) {
			if (err) {
				reject(err);
			} else {
				resolve(data);
			}
		});
	});
}

/**
 * The blank .timecard.json value.
 */

function blankTimecard(name) {
	return '{\n  "project": "' + name + '",\n  "shifts": [],\n  "totals": {\n    "hours": 0,\n    "minutes": 0,\n    "seconds": 0\n  }\n}';
}

/**
 * Check if .timecard.json is in a valid json format.
 *
 * @returns {*} Promise
 */

function checkForValidTimecard() {
	var _this = this;

	var self = this;

	return new _Promise(function callee$1$0(resolve) {
		var data, json;
		return _regeneratorRuntime.async(function callee$1$0$(context$2$0) {
			while (1) switch (context$2$0.prev = context$2$0.next) {
				case 0:
					context$2$0.next = 2;
					return _regeneratorRuntime.awrap(self.getTimeCard()['catch'](function () {
						resolve(false);
					}));

				case 2:
					data = context$2$0.sent;
					json = undefined;

					try {
						json = JSON.parse(data);
					} catch (err) {
						resolve(false);
					}
					resolve(json && json.project && json.totals && json.shifts && true);

				case 6:
				case 'end':
					return context$2$0.stop();
			}
		}, null, _this);
	});
}